@@ -11,12 +11,12 @@ import time
 from datetime import datetime
 import logging
 from unittest import TestCase
+import itertools
 
 from dateutil import parser
 
 from exoline import exo
 from exoline.exo import ExolineOnepV1
-import pytz
 from exoline import timezone
 
 try:
@@ -589,8 +589,8 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         self.assertEqual(readscript(childcik3, lua1['name']), lua2['content'], "grandchild updated")
 
 
-    def _removed_thing(self):
-        '''OneP resource usage'''
+    def usage_test(self):
+        '''Get resource usage'''
         # This test passes inconsistently due to time passing between calls to
         # usage. Mainly all it was testing was date parsing, though.
         #r = rpc('usage', self.client.cik(), '--start=10/1/2012', '--end=11/1/2013')
@@ -657,9 +657,11 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
             return int(time.mktime(t))
 
         def tolocaltz(s):
-            # parse string and translate to local timezone
+            # parse string and translate to local timezone with offset
+            # specified
             tz = timezone.localtz()
-            return str(tz.localize(datetime.utcfromtimestamp(parse_ts(s))))
+            local_time_without_offset = datetime.fromtimestamp(parse_ts(s))
+            return str(tz.localize(local_time_without_offset))
 
         self.ok(r, 'two readings on one timestamp', match='{0},,3,0.3'.format(tolocaltz('2013-07-20 03:00:08')))
 
@@ -678,6 +680,13 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         self.ok(r, 'rid order reversed')
         self.assertTrue(r.stdout == '{0},0.3,3,'.format(tolocaltz('2013-07-20 03:00:08')), 'rid order reversed')
 
+        rids = [rid for rid in itertools.chain(*json.loads(rpc('listing', cik, '--type=dataport', '--type=datarule').stdout))]
+
+        # this test could be better -- for now, just verify that the right
+        # number of columns come out
+        r = rpc('read', '--start=2013-07-20T3:00:08', '--end=2013-07-20T3:00:08', '--timeformat=unix', cik)
+        self.ok(r, 'read all RIDs', match='timestamp,' + ','.join(['.*' for rid in rids]) + '\n[0-9]+' + ','.join(['.*' for rid in rids]))
+
     def copy_diff_test(self):
         '''Copy and diff commands'''
         stdports = self._createDataports()
@@ -787,7 +796,10 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         copy_without_comments = r.stdout
 
         r = rpc('diff', childcik, copy_without_comments)
-        self.ok(r, 'no differences', match='')
+        if sys.version_info < (2, 7):
+            self.notok(r, 'diff not supported with Python <2.7')
+        else:
+            self.ok(r, 'no differences', match='')
 
         # add comments
         exo = ExolineOnepV1()
@@ -795,14 +807,21 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         exo.comment(childcik, ridFloat, 'public', 'World')
 
         r = rpc('diff', childcik, copy_without_comments)
-        self.ok(r, 'diff notices comment differences', search=r'^\+.+')
+        if sys.version_info < (2, 7):
+            self.notok(r, 'diff not supported with Python <2.7')
+        else:
+            self.ok(r, 'diff notices comment differences', search=r'^\+.+')
 
         r = rpc('copy', childcik, cik, '--cikonly')
         self.ok(r, 'make copy without comments')
         copy_with_comments = r.stdout
 
         r = rpc('diff', childcik, copy_with_comments)
-        self.ok(r, 'no differences -- comment was copied', match='')
+        if sys.version_info < (2, 7):
+            self.notok(r, 'diff not supported with Python <2.7')
+        else:
+            self.ok(r, 'no differences -- comment was copied', match='')
+
 
     def copy_limit_test(self):
         '''Check limits with copy command'''
@@ -812,17 +831,17 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         '''Connection settings'''
         cik = self.client.cik()
 
-        r = rpc('--port=80', '--host=m2.exosite.com', 'info', cik)
+        r = rpc('--port=80', '--host=m2.exosite.com', '--http', 'info', cik)
         self.ok(r, 'valid port and host at command line')
 
         r = rpc('--https', 'info', cik)
         self.ok(r, 'https connection')
 
-        r = rpc('--port=443', '--host=m2.exosite.com', '--https', 'info', cik)
+        r = rpc('--port=443', '--host=m2.exosite.com', 'info', cik)
         self.ok(r, 'invalid port')
 
         r = rpc('--port=88', 'info', cik)
-        self.notok(r, 'invalid port', match='JSON RPC Request Exception.*')
+        self.notok(r, 'wrong port', match='JSON RPC Request Exception.*')
 
     def info_test(self):
         '''Info command'''
@@ -893,6 +912,7 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         r = rpc('read', cik, rid1, '--start=44', '--timeformat=unix', '--limit=2')
         self.ok(r, '--end has a default', match='[0-9]+,12.36')
 
+
     def sort_test(self):
         '''Read command with --sort flag'''
         cik = self.client.cik()
